//
// adi_ttcp_relay.c
//
// Contains the code to execute RELAY requests.
//
// The entrypoint is TTCP_RunRelay() which accepts a connection on the RELAY's
// listening socket and then merely loops copying anything it reads from that
// connection to the remote control stream, and vice versa. Whenever one of
// the sockets closes, TTCP_RunRelay() closes the other one and then exits.
//
#include "config.h"
#include "adi_ttcp.h"

void TTCP_RunRelay(void* arg)
{
    PRELAY_BLOCK prly = arg;
    PMSG_BLOCK   pmsg = &prly->m_msgBlk;
    SOCKET       init_socket;
    SOCKADDR_IN  init_addr;
    int          init_addr_len = sizeof(init_addr);
    int          num_fd = 0;
    char         buf[32];

    // await connection from initiating end of RELAY command
    init_socket = accept(prly->m_listen_socket,
                     (PSOCKADDR)&init_addr,
                     &init_addr_len);

    if (init_socket == SOCKET_ERROR)
    {
	TTCP_ConsoleMsg("ERROR: failed to accept RELAY connection (%d)\r\n",
	                TTCP_GetLastError());
	return;
    }

    TTCP_ConsoleMsg("RELAY connection accepted from %s\r\n",
                    TTCP_GetAddr(&init_addr, buf));

    // close the listening socket
    closesocket(prly->m_listen_socket);

#ifndef _WIN32
    // gets a warning on WIN32 since sockets are not simple integers
    // (value not needed anyway - WINSOCK2's select() ignores first argument)
    num_fd = ((init_socket > prly->m_remote_socket) ?
                                        init_socket : prly->m_remote_socket) + 1;
#endif
    // loop awaiting input on one or both of the sockets
    while (1)
    {
	fd_set read_set;

	FD_ZERO(&read_set);
	FD_SET(init_socket, &read_set);
	FD_SET(prly->m_remote_socket, &read_set);

	select(num_fd, &read_set, NULL, NULL, NULL);

	if (FD_ISSET(init_socket, &read_set))
	{
	    char buf[128];

	    int n = recv(init_socket, buf, sizeof(buf), 0);

	    if (n > 0)
	    {
		send(prly->m_remote_socket, buf, n, 0);
	    }
	    else
	    {
		closesocket(init_socket);
		closesocket(prly->m_remote_socket);
		break;
	    }
	}

	if (FD_ISSET(prly->m_remote_socket, &read_set))
	{
	    char buf[128];

	    int n = recv(prly->m_remote_socket, buf, sizeof(buf), 0);

	    if (n > 0)
	    {
		send(init_socket, buf, n, 0);
	    }
	    else
	    {
		closesocket(init_socket);
		closesocket(prly->m_remote_socket);
		break;
	    }
	}
    }

    TTCP_ConsoleMsg("RELAY connection from %s closed\r\n",
                    TTCP_GetAddr(&init_addr, buf));
}
